-- Copyright (c) 2012 Cloudera, Inc. All rights reserved.
--
-- Version 35 of the CMF schema. Migration code to be applied on top of the
-- previous version.
--

update SCHEMA_VERSION set VERSION=35;

--
-- Remove the non-null constraint for the service id associated with a config
-- revision since a revision can be tied to either a service or a host.
-- Create a new column in CONFIG_REVISIONS to store host id
--
alter table CONFIG_REVISIONS
      modify SERVICE_ID bigint;

alter table CONFIG_REVISIONS
      add column HOST_ID bigint;

--
-- Remove the non-null constraint for the service id associated with a config
-- since a config can be tied to either a service or a host.
-- Create a new column in CONFIGS to store host id
--
alter table CONFIGS
      modify SERVICE_ID bigint;

alter table CONFIGS
      add column HOST_ID bigint;

--
-- Each host with configs must be tied to a configuration revision. When the
-- configs are modified, a new revision is created and all non-inherited
-- configs are added to it.
-- Create a new column in HOSTS to refer to CONFIG_REVISIONS
--
alter table HOSTS
      add column CONFIG_REVISION_ID bigint;

--
-- Insert a base revision for each existing host.
-- Automatic incrementing of the revision ID is temporarily enabled to make it
-- easier to create a new revision.
-- After inserting revisions for each host, auto incrementing is turned off.
--
alter table CONFIG_REVISIONS
      modify column REVISION_ID bigint not null auto_increment;

insert into CONFIG_REVISIONS (HOST_ID, CREATED_INSTANT, MESSAGE)
      select HOST_ID, unix_timestamp(now()), 'Base revision'
      from HOSTS h;

alter table CONFIG_REVISIONS
      modify column REVISION_ID bigint not null;

--
-- Link each host to its current config revision.
--
update HOSTS h
      set CONFIG_REVISION_ID=
      (select REVISION_ID
      from CONFIG_REVISIONS r
      where h.HOST_ID=r.HOST_ID);

--
-- All data insertion is done. Set up indices and constraints.
--
alter table CONFIGS
      add index IDX_CONFIG_HOST (HOST_ID),
      add constraint FK_CONFIG_HOST
      foreign key (HOST_ID)
      references HOSTS (HOST_ID);

alter table CONFIG_REVISIONS
      add index IDX_CONFIG_REVISIONS_HOST (HOST_ID),
      add constraint FK_CONFIG_REVISION_HOST
      foreign key (HOST_ID)
      references HOSTS (HOST_ID);

alter table HOSTS
      add index IDX_HOST_CONFIG_REVISION (CONFIG_REVISION_ID),
      add constraint FK_HOST_CONFIG_REVISION
      foreign key (CONFIG_REVISION_ID)
      references CONFIG_REVISIONS (REVISION_ID);
