// Copyright (c) 2011-2012 Cloudera, Inc.  All rights reserved.
define(["cloudera/Util"], function(Util) {
	return function(options) {
		var self = this;

		// None of these selections are empty by default (need to handle the back button).
		// The selected version.
		var version = $("input[name=serviceVersion][type=radio]:checked").val() ||
			$("input[name=serviceVersion][type=hidden]").val() || "";
		self.selectedCDHVersion = ko.observable(version);

		// The selected package.
		var pkg = $("input[name=package]:checked").val() || "";
		self.selectedPackage = ko.observable(pkg);

		// The selected service types.
		var serviceTypes = [];
		$("#serviceSelector input[name=serviceType]:checked").each(function (i, elem) {
			serviceTypes.push($(elem).val());
		});
		self.selectedServiceTypes = ko.observableArray(serviceTypes);

		self.continueSubmitted = ko.observable(false);
		self.inspectSubmitted = ko.observable(false);

		/**
		 * Returns true if the Custom package is selected.
		 */
		self.customPackageSelected = ko.dependentObservable(function() {
			return self.selectedPackage() === options.customPackage;
		}, self);

		/**
		 * Returns true if selection on this page is complete.
		 */
		self.isSelectionComplete = ko.dependentObservable(function () {
			var cdhVersion = self.selectedCDHVersion();
			var pkg = self.selectedPackage();
			var serviceTypes = self.selectedServiceTypes();
			var result = (cdhVersion !== "") && (pkg !== "") &&
				(!self.customPackageSelected() || serviceTypes.length > 0);
			return result;
		}, self);

		/**
		 * Returns true if the Continue button should be enabled.
		 */
		self.isContinueEnabled = ko.dependentObservable(function () {
			return !self.continueSubmitted() && self.isSelectionComplete();
		}, self);

		/**
		 * Returns true if the Inspect Role Assignments button should be enabled.
		 */
		self.isInspectEnabled = ko.dependentObservable(function () {
			return !self.inspectSubmitted() && self.isSelectionComplete();
		}, self);

		/**
		 * Click on the inspect button.
		 */
		self.inspectClicked = function (viewModel, e) {
			e.preventDefault();
			if (this.inspectSubmitted()) {
				this.inspectSubmitted(true);
				return false;
			}
			$('#continueForm').prop('action', 'select-hosts').submit();
		};

		/**
		 * Click on the Continue button.
		 */
		self.continueClicked = function (viewModel, e) {
			e.preventDefault();
			if (this.continueSubmitted()) {
				this.continueSubmitted(true);
				return false;
			}
			$('#continueForm').prop('action', 'autoconfig').submit();
		};

		/**
		 * Click on a package option.
		 */
		self.packageClicked = function (context, e) {
			var $radiobtn = $(e.currentTarget);
			self.selectedPackage($radiobtn.attr("value"));
			return true;
		};

		/**
		 * Click on a service type.
		 */
		self.serviceTypeClicked = function (context, e) {
			var $checkbox = $(e.currentTarget);
			if ($checkbox.attr("disabled") !== null) {
				var serviceType = $checkbox.attr("value");
				self.toggleServiceType(serviceType);
				return true;
			}
		};

		/**
		 * Toggle the selection of a service type.
		 */
		self.toggleServiceType = function(serviceType) {
			var i;
			var serviceTypes = self.selectedServiceTypes();
			for (i = 0; i < serviceTypes.length; i += 1) {
				if (serviceTypes[i] === serviceType) {
					self.selectedServiceTypes.splice(i, 1);
					return;
				}
			}
			self.selectedServiceTypes.splice(0, 0, serviceType);
		};

		/**
		 * When a different CDH Version is clicked,
		 * remove all current custom service type selection.
		 */
		self.cdhVersionClicked = function (context, e) {
			var $radio = $(e.currentTarget);
			var cdhVersion = $radio.attr("value");
			if (self.selectedCDHVersion() !== cdhVersion) {
				self.selectedCDHVersion(cdhVersion);
				self.selectedServiceTypes.removeAll();
				$("#serviceSelector").find("input").removeAttr("checked");
			}
			return true;
		};
	};
});
